## --------------------------------------------------------------------------
## freetrack - Python Script
## --------------------------------------------------------------------------
##
## DESCRIPTION:
##	A tool for streaming and recording FreeTrack data.
##
## USAGE (PYTHON):
##	import freetrack
##
## AUTHOR:
##	Christian Staschinski - info@staschi.com - www.staschi.com
##
## COPYRIGHT:
##	Copyright (c) 2014 Christian Staschinski - All Rights Reserved.
##
## --------------------------------------------------------------------------
## http://www.staschi.com/script/dualview/
## --------------------------------------------------------------------------

import maya.cmds as mc

from maya.utils import executeDeferred
from threading import Thread, Event
from _winreg import HKEY_CURRENT_USER, OpenKey, QueryValueEx, CloseKey
from ctypes import WinDLL, Structure, pointer, c_int, c_float, c_bool
from platform import architecture



# --------------------------------------------------------------------------
# Thread Class
# --------------------------------------------------------------------------
class thread(Thread):
	def __init__(self, runTime, command):
		self.runTime = runTime
		self.command = command
		self.threadLoop = True
		self.tmpEvent = Event()
		Thread.__init__(self)
	def run(self):
		while self.threadLoop:
			self.tmpEvent.wait(self.runTime)
			executeDeferred(self.command)



# --------------------------------------------------------------------------
# Data Class
# --------------------------------------------------------------------------
class data(Structure):
	_fields_= [('DataID', c_int),
               ('CamWidth', c_int),
               ('CamHeight', c_int),
               ('Yaw', c_float),
               ('Pitch', c_float),
               ('Roll', c_float),
               ('X', c_float),
               ('Y', c_float),
               ('Z', c_float),
               ('RawYaw', c_float),
               ('RawPitch', c_float),
               ('RawRoll', c_float),
               ('RawX', c_float),
               ('RawY', c_float),
               ('RawZ', c_float),
               ('X1', c_float),
               ('Y1', c_float),
               ('X2', c_float),
               ('Y2', c_float),
               ('X3', c_float),
               ('Y3', c_float),
               ('X4', c_float),
               ('Y4', c_float)]



# --------------------------------------------------------------------------
# Reader Class
# --------------------------------------------------------------------------
class reader(object):
	debug = False
	absoluteTransform = True
	worldSpaceTransform = True
	offsetTransform = True
	frameRate = 0.040
	selectedObject = ''
	expand = True
	recording = False
	count = 4
	recordMinTime = 0
	recordMaxTime = 0
	readerData = [0.000]*6
	previousData = [0.000]*6
	yawMultiply = 1.000
	pitchMultiply = 1.000
	rollMultiply = 1.000
	xMultiply = 0.010
	yMultiply = 0.010
	zMultiply = 0.010
	yawAdd = 0.000
	pitchAdd = 0.000
	rollAdd = 0.000
	xAdd = 0.000
	yAdd = 0.000
	zAdd = 0.000



	# ----------------------------------------------------------------------
	# Init Procedure
	# ----------------------------------------------------------------------
	def __init__(self):
		self.createUI()



	# ----------------------------------------------------------------------
	# Create UI Procedure
	# ----------------------------------------------------------------------
	def createUI(self):
		if not mc.dockControl('ft_dock', query=True, exists=True):
			self.scrollLayoutLayout = mc.scrollLayout ('scrollLayoutLayout', width=248)
			self.dockName = mc.dockControl('ft_dock', label='freetrack', area='right', content=self.scrollLayoutLayout, allowedArea=('right', 'left'))
			self.buttonsColumnLayout = mc.columnLayout('ft_buttonsColumnLayout', width=240, adjustableColumn=True)
			self.connectOrDisconnectButton = mc.button('ft_connectOrDisconnectButton', command=self.connectOrDisconnect, label='Disconnected', backgroundColor=[0.75,1,0.75])
			self.activeOrInactiveObjectButton = mc.button('ft_activeOrInactiveObjectButton', command=self.activeOrInactive, label='Inactive', backgroundColor=[1,1,0.75], visible=False)
			self.recordOrStopButton = mc.button('ft_recordOrStopButton', command=self.recordOrStop,label='Stopped', backgroundColor=[1,0.75,0.75], visible=False)
			mc.setParent( '..' )
			self.settingsFrameLayout = mc.frameLayout('ft_settingsFrameLayout', label='Settings', width=240, collapsable=True, collapse=False)
			self.settingsColumnLayout = mc.columnLayout('ft_settingsColumnLayout', adjustableColumn=True)
			self.transformRadioButtonGrp = mc.radioButtonGrp('ft_transformRadioButtonGrp', adjustableColumn3=3, columnWidth3=(60,60,10), select=1, onCommand1=self.absoluteTransformToogle, offCommand1=self.absoluteTransformToogle, labelArray2=['absolute', 'relative'], numberOfRadioButtons=2, label='Transform')
			self.spaceRadioButtonGrp = mc.radioButtonGrp('ft_spaceRadioButtonGrp', adjustableColumn3=3, columnWidth3=(60,60,10), select=1, onCommand1=self.worldSpaceTransformToggle, offCommand1=self.worldSpaceTransformToggle, labelArray2=['World', 'Object'], numberOfRadioButtons=2, label='Space')
			self.offsetRadioButtonGrp = mc.radioButtonGrp('ft_offsetButtonGrp', adjustableColumn3=3, columnWidth3=(60,60,10), select=1, onCommand1=self.offsetTransformToggle, offCommand1=self.offsetTransformToggle, labelArray2=['Current', 'None'], numberOfRadioButtons=2, label='Offset')
			self.frameRateField = mc.intFieldGrp('ft_frameRateField', adjustableColumn3=3, columnWidth3=(60,50,50), label='Frame Rate', extraLabel='fps', value1=25, numberOfFields=1, changeCommand=self.setFrameRate)
			if self.debug:
				self.camWidthField = mc.intFieldGrp('ft_camWidthField', adjustableColumn3=3, columnWidth3=(60,50,50), label='Cam Width', extraLabel='px', numberOfFields=1, enable1=False)
				self.camHeightField = mc.intFieldGrp('ft_camHeightField', adjustableColumn3=3, columnWidth3=(60,50,50), label='Cam Height', extraLabel='px', numberOfFields=1, enable1=False)
			mc.setParent( '..' )
			mc.setParent( '..' )
			self.dataFrameLayout = mc.frameLayout('ft_dataFrameLayout', label='Data', width=240, collapsable=True, collapse=False, collapseCommand=self.expandToggle, expandCommand=self.expandToggle)
			self.dataColumnLayout = mc.columnLayout('ft_dataColumnLayout', adjustableColumn=True)
			self.yawField = mc.floatFieldGrp('ft_yawField', columnWidth4=(60,50,50,50), label='Yaw', value1=0.000, value2=1, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.pitchField = mc.floatFieldGrp('ft_pitchField', columnWidth4=(60,50,50,50), label='Pitch', value1=0.000, value2=1, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.rollField = mc.floatFieldGrp('ft_rollField', columnWidth4=(60,50,50,50), label='Roll', value1=0.000, value2=1, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.xField = mc.floatFieldGrp('ft_xField', columnWidth4=(60,50,50,50), label='X', value1=0.000, value2=0.010, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.yField = mc.floatFieldGrp('ft_yField', columnWidth4=(60,50,50,50), label='Y', value1=0.000, value2=0.010, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.zField = mc.floatFieldGrp('ft_zField', columnWidth4=(60,50,50,50), label='Z', value1=0.000, value2=0.010, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3, changeCommand=self.setMultiply)
			self.resetButton = mc.button('ft_resetButton', command=self.resetVariables, label='Reset')
			if self.debug:
				self.rawYawField = mc.floatFieldGrp('ft_rawYawField', columnWidth4=(60,50,50,50), label='Raw Yaw', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.rawPitchField = mc.floatFieldGrp('ft_rawPitchField', columnWidth4=(60,50,50,50), label='Raw Pitch', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.rawRollField = mc.floatFieldGrp('ft_rawRollField', columnWidth4=(60,50,50,50), label='Raw Roll', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.rawXfield = mc.floatFieldGrp('ft_rawXfield', columnWidth4=(60,50,50,50), label='Raw X', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.rawYfield = mc.floatFieldGrp('ft_rawYfield', columnWidth4=(60,50,50,50), label='Raw Y', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.rawZfield = mc.floatFieldGrp('ft_rawZField', columnWidth4=(60,50,50,50), label='Raw Z', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.x1Field = mc.floatFieldGrp('ft_x1Field', columnWidth4=(60,50,50,50), label='X1', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.y1Field = mc.floatFieldGrp('ft_y1Field', columnWidth4=(60,50,50,50), label='Y1', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.x2Field = mc.floatFieldGrp('ft_x2Field', columnWidth4=(60,50,50,50), label='X2', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.y2Field = mc.floatFieldGrp('ft_y2Field', columnWidth4=(60,50,50,50), label='Y2', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.x3Field = mc.floatFieldGrp('ft_x3Field', columnWidth4=(60,50,50,50), label='X3', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.y3Field = mc.floatFieldGrp('ft_y3Field', columnWidth4=(60,50,50,50), label='Y3', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.x4Field = mc.floatFieldGrp('ft_x4Field', columnWidth4=(60,50,50,50), label='X4', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
				self.y4Field = mc.floatFieldGrp('ft_y4Field', columnWidth4=(60,50,50,50), label='Y4', value1=0.000, value2=0.000, value3=0.000, numberOfFields=3, enable1=False, enable3=False, precision=3)
			mc.setParent( '..' )
			mc.setParent( '..' )
			self.miscFrameLayout = mc.frameLayout('ft_miscFrameLayout', label='Misc.', width=240, collapsable=True, collapse=True)
			self.miscColumnLayout = mc.columnLayout('ft_miscColumnLayout', adjustableColumn=True)
			self.helpIconTextButton = mc.iconTextButton( style='textOnly', label='www.staschi.com', command=self.loadHelpWebsite)
			mc.setParent( '..' )
			mc.setParent( '..' )
		elif not mc.dockControl('ft_dock', query=True, visible=True):
			mc.dockControl('ft_dock', edit=True, visible=True)



	# ----------------------------------------------------------------------
	# Load Dll Procedure
	# ----------------------------------------------------------------------
	def loadDll(self):
		clientLocation = 'Software\FreeTrack\FreeTrackClient'
		clientFilename = 'FreeTrackClient.dll'
		platformArchitecture = architecture()
		if (platformArchitecture[0] == '64bit'):
			clientFilename = 'FreeTrackClient64bit.dll'
		aKey = OpenKey(HKEY_CURRENT_USER,clientLocation)
		if QueryValueEx(aKey,'Path'):
			self.client = WinDLL(QueryValueEx(aKey,'Path')[0]+clientFilename)
		CloseKey(aKey)



	# ----------------------------------------------------------------------
	# Toggle Procedures
	# ----------------------------------------------------------------------
	def connectOrDisconnect(self, *args):
		if not hasattr(self,'client'): 
			self.loadDll()
			self.pointData()
		if not hasattr(self,'dataThread'):
			if mc.undoInfo( query=True, state=True ): mc.undoInfo( state=False )
			mc.button(self.connectOrDisconnectButton, edit=True, label='Connected', backgroundColor=[0,1,0])
			mc.button(self.activeOrInactiveObjectButton, edit=True, visible=True)
			self.currentAngleUnit = mc.currentUnit( query=True, angle=True )
			if (self.currentAngleUnit == 'deg'): mc.currentUnit( angle='rad' )
			self.dataThread = thread(self.frameRate, self.sendData)
			self.dataThread.setName('ft_dataThread')
			self.dataThread.start()
		else:
			if mc.undoInfo( query=True, state=False ): mc.undoInfo( state=True )
			self.dataThread.threadLoop = False
			self.selectedObject = ''
			mc.currentUnit( angle=self.currentAngleUnit )
			mc.button(self.connectOrDisconnectButton, edit=True, label='Disconnected', backgroundColor=[0.75,1,0.75])
			mc.button(self.activeOrInactiveObjectButton, edit=True, label='Inactive', backgroundColor=[1,1,0.75], visible=False)
			mc.button(self.recordOrStopButton, edit=True, visible=False)
			del self.dataThread
	def activeOrInactive(self, *args):
		if not self.selectedObject:
			self.setActive()
		else:
			self.setInactive()
	def setActive(self):
		self.selectedObject = mc.ls( selection=True, objectsOnly=True )
		if self.selectedObject:
			mc.frameLayout(self.dataFrameLayout, edit=True, collapse=True)
			self.expand = False
			mc.button(self.activeOrInactiveObjectButton, edit=True, label='Active', backgroundColor=[1,1,0])
			mc.button(self.recordOrStopButton, edit=True, visible=True)
			self.selectedObject = self.selectedObject[0]
			self.updateAdd()
			self.setAdd()
		else:
			self.setInactive()
	def setInactive(self):
		mc.frameLayout(self.dataFrameLayout, edit=True, collapse=False)
		self.expand = True
		mc.button(self.activeOrInactiveObjectButton, edit=True, label='Inactive', backgroundColor=[1,1,0.75])
		mc.button(self.recordOrStopButton, edit=True, visible=False)
		self.selectedObject = ''
	def recordOrStop(self, *args):
		if not self.recording:
			self.countdownThread = thread(1, self.countdownRecord)
			self.countdownThread.setName('ft_countdownThread')
			self.countdownThread.start()
		else:
			mc.button(self.recordOrStopButton, edit=True, label='Stopped', backgroundColor=[1,0.75,0.75])
			self.recording = False
	def absoluteTransformToogle(self, *args):
		self.absoluteTransform = not self.absoluteTransform
	def	worldSpaceTransformToggle(self, *args):
		self.worldSpaceTransform = not self.worldSpaceTransform
	def offsetTransformToggle(self, *args):
		self.offsetTransform = not self.offsetTransform
		self.setAdd()
	def expandToggle(self, *args):
		self.expand = not self.expand



	# ----------------------------------------------------------------------
	# Data Procedures
	# ----------------------------------------------------------------------
	def pointData(self):
		self.PFreeTrackData = pointer(data())
		self.client.FTGetData.restype = c_bool
		self.client.FTGetData.argtype = [data]
	def sendData(self):
		if self.client.FTGetData(self.PFreeTrackData):
			self.readerData = [round(self.PFreeTrackData.contents.Pitch,3), round(self.PFreeTrackData.contents.Yaw,3), round(self.PFreeTrackData.contents.Roll,3), round(self.PFreeTrackData.contents.X,3), round(self.PFreeTrackData.contents.Y,3), round(self.PFreeTrackData.contents.Z,3)]
			if self.readerData != self.previousData:
				if self.selectedObject:
					mc.xform(self.selectedObject,rotation=(self.pitchAdd+(self.readerData[0]*self.pitchMultiply), self.yawAdd+(self.readerData[1]*self.yawMultiply), self.rollAdd+(self.readerData[2]*self.rollMultiply)), translation=(self.xAdd+(self.readerData[3]*self.xMultiply), self.yAdd+(self.readerData[4]*self.yMultiply), self.zAdd+(self.readerData[5]*self.zMultiply)), absolute=self.absoluteTransform, relative=not(self.absoluteTransform), objectSpace=not(self.worldSpaceTransform), worldSpace=self.worldSpaceTransform)
				if not self.recording: mc.refresh(force=True,currentView=True)
				self.previousData = self.readerData
				if self.expand: self.updateData()
			if self.recording: self.record()
	def updateData(self):
		if not self.debug:
			mc.floatFieldGrp(self.yawField, edit=True, value1=self.previousData[0] )
			mc.floatFieldGrp(self.pitchField, edit=True, value1=self.previousData[1] )
			mc.floatFieldGrp(self.rollField, edit=True, value1=self.previousData[2] )
			mc.floatFieldGrp(self.xField, edit=True, value1=self.previousData[3] )
			mc.floatFieldGrp(self.yField, edit=True, value1=self.previousData[4] )
			mc.floatFieldGrp(self.zField, edit=True, value1=self.previousData[5] )
		else:
			mc.floatFieldGrp(self.yawField, edit=True, value1=self.PFreeTrackData.contents.Yaw )
			mc.floatFieldGrp(self.pitchField, edit=True, value1=self.PFreeTrackData.contents.Pitch )
			mc.floatFieldGrp(self.rollField, edit=True, value1=self.PFreeTrackData.contents.Roll )
			mc.floatFieldGrp(self.xField, edit=True, value1=self.PFreeTrackData.contents.X )
			mc.floatFieldGrp(self.yField, edit=True, value1=self.PFreeTrackData.contents.Y )
			mc.floatFieldGrp(self.zField, edit=True, value1=self.PFreeTrackData.contents.Z )
			mc.intFieldGrp(self.camWidthField, edit=True, value1=self.PFreeTrackData.contents.CamWidth )
			mc.intFieldGrp(self.camHeightField, edit=True, value1=self.PFreeTrackData.contents.CamHeight )
			mc.floatFieldGrp(self.rawYawField, edit=True, value1=self.PFreeTrackData.contents.RawYaw )
			mc.floatFieldGrp(self.rawPitchField, edit=True, value1=self.PFreeTrackData.contents.RawPitch )
			mc.floatFieldGrp(self.rawRollField, edit=True, value1=self.PFreeTrackData.contents.RawRoll )
			mc.floatFieldGrp(self.rawXfield, edit=True, value1=self.PFreeTrackData.contents.RawX )
			mc.floatFieldGrp(self.rawYfield, edit=True, value1=self.PFreeTrackData.contents.RawY )
			mc.floatFieldGrp(self.rawZfield, edit=True, value1=self.PFreeTrackData.contents.RawZ )
			mc.floatFieldGrp(self.x1Field, edit=True, value1=self.PFreeTrackData.contents.X1 )
			mc.floatFieldGrp(self.y1Field, edit=True, value1=self.PFreeTrackData.contents.Y1 )
			mc.floatFieldGrp(self.x2Field, edit=True, value1=self.PFreeTrackData.contents.X2 )
			mc.floatFieldGrp(self.y2Field, edit=True, value1=self.PFreeTrackData.contents.Y2 )
			mc.floatFieldGrp(self.x3Field, edit=True, value1=self.PFreeTrackData.contents.X3 )
			mc.floatFieldGrp(self.y3Field, edit=True, value1=self.PFreeTrackData.contents.Y3 )
			mc.floatFieldGrp(self.x4Field, edit=True, value1=self.PFreeTrackData.contents.X4 )
			mc.floatFieldGrp(self.y4Field, edit=True, value1=self.PFreeTrackData.contents.Y4 )



	# ----------------------------------------------------------------------
	# Add Procedures
	# ----------------------------------------------------------------------
	def setAdd(self):
		if self.offsetTransform:
			self.yawAdd = mc.floatFieldGrp(self.yawField, query=True, value3=True)
			self.pitchAdd = mc.floatFieldGrp(self.pitchField, query=True, value3=True)
			self.rollAdd = mc.floatFieldGrp(self.rollField, query=True, value3=True)
			self.xAdd = mc.floatFieldGrp(self.xField, query=True, value3=True)
			self.yAdd = mc.floatFieldGrp(self.yField, query=True, value3=True)
			self.zAdd = mc.floatFieldGrp(self.zField, query=True, value3=True)
		else:
			self.yawAdd = 0.000
			self.pitchAdd = 0.000
			self.rollAdd = 0.000
			self.xAdd = 0.000
			self.yAdd = 0.000
			self.zAdd = 0.000
	def updateAdd(self):
		mc.floatFieldGrp(self.yawField, edit=True, value3=mc.getAttr(self.selectedObject+'.rotateY'))
		mc.floatFieldGrp(self.pitchField, edit=True, value3=mc.getAttr(self.selectedObject+'.rotateX'))
		mc.floatFieldGrp(self.rollField, edit=True, value3=mc.getAttr(self.selectedObject+'.rotateZ'))
		mc.floatFieldGrp(self.xField, edit=True, value3=mc.getAttr(self.selectedObject+'.translateX'))
		mc.floatFieldGrp(self.yField, edit=True, value3=mc.getAttr(self.selectedObject+'.translateY'))
		mc.floatFieldGrp(self.zField, edit=True, value3=mc.getAttr(self.selectedObject+'.translateZ'))



	# ----------------------------------------------------------------------
	# Multiply Procedures
	# ----------------------------------------------------------------------
	def setMultiply(self, *args):
		self.yawMultiply = mc.floatFieldGrp( self.yawField, query=True, value2=True)
		self.pitchMultiply = mc.floatFieldGrp( self.pitchField, query=True, value2=True)
		self.rollMultiply = mc.floatFieldGrp( self.rollField, query=True, value2=True)
		self.xMultiply = mc.floatFieldGrp( self.xField, query=True, value2=True)
		self.yMultiply = mc.floatFieldGrp( self.yField, query=True, value2=True)
		self.zMultiply = mc.floatFieldGrp( self.zField, query=True, value2=True)
	def updateMultiply(self):
		mc.floatFieldGrp(self.yawField, edit=True, value2=self.yawMultiply)
		mc.floatFieldGrp(self.pitchField, edit=True, value2=self.pitchMultiply)
		mc.floatFieldGrp(self.rollField, edit=True, value2=self.rollMultiply)
		mc.floatFieldGrp(self.xField, edit=True, value2=self.xMultiply)
		mc.floatFieldGrp(self.yField, edit=True, value2=self.yMultiply)
		mc.floatFieldGrp(self.zField, edit=True, value2=self.zMultiply)



	# ----------------------------------------------------------------------
	# FrameRate Procedures
	# ----------------------------------------------------------------------
	def setFrameRate(self, *args):
		self.frameRate = 1.0/mc.intFieldGrp( self.frameRateField, query=True, value1=True)
	def updateFrameRate(self):
		mc.intFieldGrp( self.frameRateField, edit=True, value1=self.frameRate*625)



	# ----------------------------------------------------------------------
	# Reset Variables Procedure
	# ----------------------------------------------------------------------
	def resetVariables(self, *args):
		self.readerData = [0.000]*6
		self.previousData = [0.000]*6
		self.yawMultiply = 1.000
		self.pitchMultiply = 1.000
		self.rollMultiply = 1.000
		self.xMultiply = 0.010
		self.yMultiply = 0.010
		self.zMultiply = 0.010
		self.updateMultiply()
		self.updateData()



	# ----------------------------------------------------------------------
	# Record Procedures
	# ----------------------------------------------------------------------
	def countdownRecord(self):
		self.count-=1
		mc.button(self.recordOrStopButton, edit=True, label='Countdown: '+str(self.count), backgroundColor=[1,0.1875*self.count,0.1875*self.count])
		mc.refresh()
		if self.count == 1:
			self.countdownThread.threadLoop = False
		if self.count == 0:
			mc.button(self.recordOrStopButton, edit=True, label='Recording', backgroundColor=[1,0,0])
			self.count = 4
			self.recordMinTime = mc.currentTime( query=True )
			self.recordMaxTime = mc.playbackOptions( query=True, maxTime=True )
			self.recording = True
	def record(self, *args):
		if self.recordMinTime <= self.recordMaxTime:
			mc.setKeyframe( self.selectedObject, attribute='rotate')
			mc.setKeyframe( self.selectedObject, attribute='translate')
			self.recordMinTime += 1
			mc.currentTime( self.recordMinTime, edit=True )
		else: self.recordOrStop()



	# ----------------------------------------------------------------------
	# Load Help Website Procedure
	# ----------------------------------------------------------------------
	def loadHelpWebsite(self, *args):
		mc.showHelp( 'http://www.staschi.com/script/freetrack/', absolute=True )



reader()